package de.kompf.jna;

import java.nio.*;
import java.util.Arrays;

import com.sun.jna.WString;
import com.sun.jna.platform.win32.*;

import de.kompf.jna.CLibrary.Group;

public class JNADemo {

  /**
   * @param args
   */
  public static void main(String[] args) {
    JNADemo demo = new JNADemo();

    if (System.getProperty("os.name").startsWith("Windows")) {
      demo.printProcessId();
      demo.printVolumeInformation("C:\\");
    } else {
      demo.printProcessIdUnix();
      demo.printGroupInformation();
    }
  }

  private void printProcessId() {
    int pid = Kernel32.INSTANCE.GetCurrentProcessId();
    System.out.println("My Process id is " + pid);
  }

  private void printProcessIdUnix() {
    int pid = CLibrary.INSTANCE.getpid();
    System.out.println("My Process id is " + pid);
  }

  private void printVolumeInformation(String path) {
    final WinDef.DWORD bufSize = new WinDef.DWORD(512);
    CharBuffer volName = CharBuffer.allocate(bufSize.intValue());
    WinDef.DWORDByReference volSerNbr = new WinDef.DWORDByReference();
    WinDef.DWORDByReference maxCompLen = new WinDef.DWORDByReference();
    WinDef.DWORDByReference fileSysFlags = new WinDef.DWORDByReference();
    CharBuffer fsName = CharBuffer.allocate(bufSize.intValue());
    boolean ok = Kernel32Ext.INSTANCE.GetVolumeInformationW(new WString(path),
        volName, bufSize, volSerNbr, maxCompLen, fileSysFlags, fsName, bufSize);
    if (ok) {
      System.out.printf("%s has volume name %s and id %x%n", path,
          volName.toString().trim(), volSerNbr.getValue().longValue());
    }
  }

  private void printVolumeInformationA(String path) {
    final WinDef.DWORD bufSize = new WinDef.DWORD(512);
    ByteBuffer volName = ByteBuffer.allocate(bufSize.intValue());
    WinDef.DWORDByReference volSerNbr = new WinDef.DWORDByReference();
    WinDef.DWORDByReference maxCompLen = new WinDef.DWORDByReference();
    WinDef.DWORDByReference fileSysFlags = new WinDef.DWORDByReference();
    ByteBuffer fsName = ByteBuffer.allocate(bufSize.intValue());
    boolean ok = Kernel32Ext.INSTANCE.GetVolumeInformationA(path,
        volName, bufSize, volSerNbr, maxCompLen, fileSysFlags, fsName, bufSize);
    if (ok) {
      System.out.printf("%s has volume name %s and id %x%n", path,
          new String(volName.array()).trim(), volSerNbr.getValue().longValue());
    }
  }

  private void printGroupInformation() {
    int gid = CLibrary.INSTANCE.getgid();
    Group group = CLibrary.INSTANCE.getgrgid(gid);
    System.out.printf("Your real group is %s with id %d%n", group.gr_name, gid);
    String[] members = group.gr_mem.getStringArray(0);
    System.out.printf("Other members of this group are: %s%n", Arrays.asList(members));
    
    System.out.println("Supplementary groups:");
    IntBuffer list = IntBuffer.allocate(100);
    int nGroups = CLibrary.INSTANCE.getgroups(100, list);
    for (int i = 0; i < nGroups; ++i) {
      group = CLibrary.INSTANCE.getgrgid(list.get(i));
      System.out.printf("  %s (%d): %s%n", group.gr_name, group.gr_gid,
          Arrays.asList(group.gr_mem.getStringArray(0)));
    }
  }
}
